/*******************************************************************
  Components for embedded applications builded for
  laboratory and medical instruments firmware  
 
  mo_flashbb.S - board setup startup code containing
                 hardware setup script interpreter
 
  Copyright (C) 2001-2003 by Pavel Pisa pisa@cmp.felk.cvut.cz
            (C) 2001-2003 by PiKRON Ltd. http://www.pikron.com

 *******************************************************************/

#include <sim.h>
#include <mo_flashbb.h>


#define __val2mfld(mask,val) (((mask)&~((mask)<<1))*(val)&(mask))
#define __mfld2val(mask,val) (((val)&(mask))/((mask)&~((mask)<<1)))

.section .fvector,"x"
.org	0
__fbb_vectors:
	.long	__fbb_descr-__fbb_vectors	/* start SP value */
	.long	__fbb_start-__fbb_vectors	/* start PC value */

.text
	.even
	
__fbb_descr:
	.long	__flashpb_base - .	/* offset of first parameter block */
	.long	__flashpb_size		/* length of parameter block */

__fbb_start:
	orw	#0x0700,%sr
	/* !!!!!!!!!!!!!!!!!!!!!!!! disable int7 & comp !! */
	andib	#0xf4,SIM_PEPAR
	moveb	#0x00,SIM_PFPAR
	/* !!!!!!!!!!!!!!!!!!!!!!!! must be in 2^9 Tsys !! */
	moveb	#(SIM_HME|SIM_BME|(SIM_BMT&2)),SIM_SYPCR
	/* !!!!!!!!!!!!!!!!!!!!!!!! watchdog disabled !!!! */

	leal	%pc@(__fbb_vectors,%a7),%a6
	movel	%a6,%a5			/* a6 .. __fbb_descr */
	addal	%a6@,%a5		/* a5 .. __flashpb_base */
	movel	#1,%d2

4:	movel	%a6@(4),%d1		/* d1 .. __flashpb_size */
	lsrl	#2,%d1
	subl	#2,%d1
	clrl	%d0
	movel	%a5,%a0
	tstw	%a0@
	jeq	__fbb_try_next
5:	addl	%a0@+,%d0
	dbra	%d1,5b
	subl	%a0@,%d0
	jeq	7f
__fbb_try_next:
	addl	%a6@(4),%a5
	dbra	%d2,4b
/* no walid HWCFG in any __flashpb_block */
	movel	#0,%a5
	jra	__fbb_hwcfg_default	
__fbb_halt:
	nop
	jra	__fbb_halt		/* no valid parameter block */
7:	

/* find hardware configuration block */
	movel	%a5,%a4
1:	movew	%a4@+,%d1
	movew	%a4@+,%d0
	cmpw	#FPB_ID_HWCFG,%d0
	jeq	__fbb_hwcfg_from_fpb
	tstw	%d0
	jeq	__fbb_hwcfg_default
	addw	#1,%d0
	jeq	__fbb_hwcfg_default
	addl	%d1,%a4
	jra	1b
__fbb_hwcfg_default:
	leal	%pc@(__flashbb_default_hwcfg),%a4
__fbb_hwcfg_from_fpb:
	movel	%a4,%a2			/* a4 valid HWCFG */
	leal	%pc@(__fbb_call_fbb_main),%a3
/* fall through into __fbb_hwcfg_proc */
	
/* proccess hardware configuration block	*/
/* call	: a2 .. points to HWCFG			*/
/*	  a3 .. return address			*/
__fbb_hwcfg_proc:
	movel	%a2@+,%d0
	movel	%d0,%d1
	andl	#~(FPB_HWC_MASK),%d0
	eorl	%d0,%d1
	roll	#(32-FPB_HWC_SHIFT),%d1
	movew	%pc@(__fbb_hwcfg_cmd_jt,%d1:l:2),%a0
	jmp	%pc@(__fbb_hwcfg_cmd_jb,%a0)
__fbb_hwcfg_cmd_jb:

__fbb_hwcfg_cmd_end:
	jra	%a3@

__fbb_hwcfg_cmd_jmp:
	jra	%zpc@(%d0:l)

__fbb_hwcfg_cmd_sp:
	movel	%d0,%sp
	jra	__fbb_hwcfg_proc

__fbb_hwcfg_cmd_relo:
	jra	%pc@(%d0:l)
__fbb_hwcfg_cmd_relo1:
	addl	%d0,%a2
	addl	%d0,%a3
	addl	%d0,%a4
	addl	%d0,%a5
	addl	%d0,%a6
	addl	%d0,%a7
	jra	__fbb_hwcfg_proc

__fbb_hwcfg_cmd_wrb:
	movel	%d0,%a0		/* dest addr */
	movel	%a2@+,%d0	/* data */
	moveb	%d0,%a0@	/* do byte write */
	jra	__fbb_hwcfg_proc

__fbb_hwcfg_cmd_wrs:
	movel	%d0,%a0		/* dest addr */
	movel	%a2@+,%d0	/* data */
	movew	%d0,%a0@	/* do short write */
	jra	__fbb_hwcfg_proc

__fbb_hwcfg_cmd_wrl:
	movel	%d0,%a0		/* dest addr */
	movel	%a2@+,%d0	/* data */
	movel	%d0,%a0@	/* do long write */
	jra	__fbb_hwcfg_proc

__fbb_hwcfg_cmd_wrm:
	movel	%d0,%a0		/* dest addr */
	movel	%a2@+,%d0	/* byte count */
	jmi	3f
	addl	#1,%d0
	movel	%d0,%d1
	andw	#2,%d1
	lsrl	#1,%d0
	jra	2f
1:	movew	%a2@+,%a0@+	/* transfer by short */
2:	dbra	%d0,1b
	addw	%d1,%a2	
	jra	__fbb_hwcfg_proc
3:	addl	#3,%d0
	lsrl	#2,%d0
	jra	5f
4:	movel	%a2@+,%a0@+	/* transfer by long */
5:	dbra	%d0,4b
	jra	__fbb_hwcfg_proc


__fbb_hwcfg_cmd_clrm:
	movel	%d0,%a0
	movel	%a2@+,%d0
	subl	#1,%d0
1:	clrw	%a0@+
	dbra	%d0,1b
	subl	#0x10000,%d0
	bccs	1b
	jra	__fbb_hwcfg_proc

__fbb_hwcfg_cmd_fsrel:
	movel	%a2@+,%a1	/* help mem */
	movel	%a2@+,%a0	/* register addr */
	movel	%a0,%d1
	andl	#0x0fffffff,%d1	/* mask register address */	
	exg	%a0,%d1
	tstl	%d1
	jpl	2f		/* long op ? */
	btstl	#30,%d1		/* long op */
	beqs	1f
	movel   %a0@,%d1	/* chek for already set reg */
	cmpl	%a2@,%d1
	jeq	5f
1:	movel	#0x20814ed1,%a1@
	/* movel %d1,%a0@; jmp %a1@ ; */
	jra	4f
2:	btstl	#30,%d1		/* word op */
	beqs	3f
	movew   %a0@,%d1	/* chek for already set reg */
	cmpw	%a2@(2),%d1
	jeq	5f
3:	movel	#0x30814ed1,%a1@
	/* movew %d1,%a0@; jmp %a1@ ; */
4:	movel	%a2@+,%d1	/* register data */
	movel	%a1,%d2
	leal	%pc@(__fbb_hwcfg_cmd_relo1,%d0),%a1
	jmp	%zpc@(%d2)
5:	addl	#4,%a2
	jra	__fbb_hwcfg_proc

__fbb_hwcfg_cmd_jt:
	.short	__fbb_hwcfg_cmd_end-__fbb_hwcfg_cmd_jb
	.short	__fbb_hwcfg_cmd_jmp-__fbb_hwcfg_cmd_jb
	.short	__fbb_hwcfg_cmd_sp-__fbb_hwcfg_cmd_jb
	.short	__fbb_hwcfg_cmd_relo-__fbb_hwcfg_cmd_jb
	.short	__fbb_hwcfg_cmd_wrb-__fbb_hwcfg_cmd_jb
	.short	__fbb_hwcfg_cmd_wrs-__fbb_hwcfg_cmd_jb
	.short	__fbb_hwcfg_cmd_wrl-__fbb_hwcfg_cmd_jb
	.short	__fbb_hwcfg_cmd_wrm-__fbb_hwcfg_cmd_jb
	.short	__fbb_hwcfg_cmd_clrm-__fbb_hwcfg_cmd_jb
	.short	__fbb_hwcfg_cmd_fsrel-__fbb_hwcfg_cmd_jb

/* call main of boot block support */ 
__fbb_call_fbb_main:
	movew	%pc@(__usrprog_start),%d0
	cmpw	#0xffff,%d0
	jeq	__fbb_call_fbb_main
	movel	%a5,%sp@-		/* a5 .. __flashpb_base */
	movel	%a6,%sp@-		/* a6 .. __fbb_descr */
	jmp	%pc@(__usrprog_start)

