/*******************************************************************
  Motion and Robotic System (MARS) aplication components.
 
  system_def.h - definition of SIMPLE system peripheral address map
                 and conditional components inclusion
 
  Copyright (C) 2001-2003 by Pavel Pisa - originator 
                          pisa@cmp.felk.cvut.cz
            (C) 2001-2003 by PiKRON Ltd. - originator
	                  http://www.pikron.com

 *******************************************************************/

#ifndef _SYSTEM_DEF_H_
#define _SYSTEM_DEF_H_

#include <types.h>

#define VER_CODE(major,minor,patch) (major*0x10000+minor*0x100+patch)
/* Software version */
#define SW_VER_ID	"SIMPLE"
#define SW_VER_MAJOR	0
#define SW_VER_MINOR	1
#define SW_VER_PATCH	0
#define SW_VER_CODE	VER_CODE(SW_VER_MAJOR,SW_VER_MINOR,SW_VER_PATCH)
/* Hardware version */
#define HW_VER_ID	"MO_CPU1"
#define HW_VER_MAJOR	1
#define HW_VER_MINOR	2
#define HW_VER_PATCH	0
#define HW_VER_CODE	VER_CODE(HW_VER_MAJOR,HW_VER_MINOR,HW_VER_PATCH)
/* Version of mechanical  */
#define MECH_VER_ID	"none"
#define MECH_VER_MAJOR  0
#define MECH_VER_MINOR  0
#define MECH_VER_PATCH  0
#define MECH_VER_CODE	VER_CODE(MECH_VER_MAJOR,MECH_VER_MINOR,MECH_VER_PATCH)

#define CPU_REF_HZ  4000000	/* reference clock frequency      */
#define CPU_SYS_HZ 21000000	/* default system clock frequency */

int cpu_ref_hz;			/* actual external XTAL reference */
int cpu_sys_hz;			/* actual system clock frequency  */

#define ISR_SIM_IARB   0xf

#define ISR_QSM_IARB   0xe
#define ISR_QSM_ILSCI  4
#define ISR_QSM_INTV   0x60	/* 2x .. SCI, SPI */

/* PWM and sampling frequency */
#define ISR_CTM4_IARB  0xd
#define ISR_CTM4_IL    3
#define ISR_CTM4_INTV  0x80	/* 64x .. 13 used */
#define ISR_CTM4_SFI_INTV (0x80+2)  /* MCSM2 */
#define ISR_CTM4_SFI_IL   5
#define CTM4_SFI_HZ    1000	/* MCSM2 sampling frequency */
unsigned ctm4_sfi_hz;		/* actual sampling frequency */
volatile unsigned msec_time;	/* system time in msec units */

/* TouCAN subsystem configuration */
#define ISR_CAN_IARB  0xc
#define ISR_CAN_IL    6
#define ISR_CAN_INTV  0xa0	/* 32x .. 19 used */

/* queued ACD module */
#define ISR_QADC_IARB	0xb

/* IIC interface PCD8584 */
#define PCD_8584_PORT0 (volatile __u8 * const)(0xf10000)
#define PCD_8584_PORT1 (volatile __u8 * const)(0xf10001)
#define PCD_8584_CSOR  SIM_CSOR4

/* Keyboard KL41 */
#define KL41_LCD_INST  (volatile __u8 * const)(0xf89000)
#define KL41_LCD_STAT  (volatile __u8 * const)(0xf89001)
#define KL41_LCD_WDATA (volatile __u8 * const)(0xf89002)
#define KL41_LCD_RDATA (volatile __u8 * const)(0xf89003)
#define KL41_LED_WR    (volatile __u8 * const)(0xf89001)
#define KL41_KBD_WR    (volatile __u8 * const)(0xf89003)
#define KL41_KBD_RD    (volatile __u8 * const)(0xf89004)

#if 1
/* SGM Small graphics LCD module 240x64 */
#define SGM_LCD_DATA  (volatile __u8 * const)(0xf89000)
#define SGM_LCD_CMD   (volatile __u8 * const)(0xf89001)
#define SGM_LCD_STAT  (volatile __u8 * const)(0xf89001)
/* Keyboard on MO_KBD1 */
#define SGM_KBDI      (volatile __u8 * const)(0xf89002)
#define SGM_KBDO      (volatile __u8 * const)(0xf89002)
#endif

/* Enable RS232 GDB debugging */
#define WITH_GDBSTUB
#define WITHOUT_GDBSTUB_HOOKS
#define GDBSTUB_DATA_SECT __attribute((section (".gdbdata")))
/* requires WITH_GDBSTUB and WITHOUT_GDBSTUB_HOOKS */
/* enables to download and test application uder FLASH serial stub */
#undef GDBSTUB_RAM_TARGET

/* RS232 defaults */
/*#define WITH_RS232_ECHO_OFF*/

#endif /* _SYSTEM_DEF_H_ */
